import { AfterContentInit, AfterViewInit, Component, ElementRef, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import Picker from 'vanilla-picker';
import { MatButtonModule } from '@angular/material/button';
import { Subject, delay, startWith } from 'rxjs';

export interface Color {
  rgba: number[];
  hsla: number[];
  rgbString: string;
  rgbaString: string;
  hslString: string;
  hslaString: string;
  hex: string;
}

@Component({
  selector: 'color-picker',
  templateUrl: './color-picker.component.html',
  styleUrls: ['./color-picker.component.scss'],
})
export class ColorPickerComponent implements AfterViewInit {

  @ViewChild('clickColorPickerElement')
  clickColorPickerElement?: ElementRef;

  @Output('color')
  outColor = new EventEmitter<Color>;

  @Input('initialColor')
  initialColor?: string;

  loaded: Subject<void> = new Subject<void>();

  picker?: Picker;
  showPicker: boolean = false;

  constructor() {
    this.loaded.pipe(delay(0)).subscribe(() => {
      this.picker = new Picker({
        onChange: this.outColor.next.bind(this.outColor),
        parent: this.clickColorPickerElement?.nativeElement,
        color: this.initialColor
      });
    })
  }

  ngAfterViewInit(): void {
    this.loaded.next();
  }
}
