import { Component, OnInit } from '@angular/core';

import {
  CdkDrag,
  CdkDragDrop,
  CdkDropList,
  CdkDropListGroup,
  moveItemInArray,
} from '@angular/cdk/drag-drop';

import { CommonModule } from '@angular/common';
import { WorldArea } from '../models/world-area';
import { Plan, PlanElement } from '../models/plan';
import { WorldAreaService } from '../services/world-area.service';
import { FormsModule } from '@angular/forms';
import { Fuzzr } from '../fuzzr/fuzzr';

@Component({
  selector: 'plan-editor',
  templateUrl: './editor.component.html',
  styleUrls: ['./editor.component.scss'],
  standalone: true,
  imports: [
    CommonModule,
    FormsModule,
    CdkDropListGroup,
    CdkDropList,
    CdkDrag
  ],
})
export class EditorComponent implements OnInit {
  areas?: WorldArea[];
  planAreas: WorldArea[];
  plan: Plan;
  areasMap?: Map<String, WorldArea>;
  areaSearchString: string = "";
  planSearchString: string = "";
  planFuzzer: Fuzzr;

  constructor(public worldAreaService: WorldAreaService) {
    this.plan = {
      plan: [],
      current: 0,
    }

    this.planFuzzer = new Fuzzr(this.plan.plan, {
      toString: (e: PlanElement) => {
        return this.areasMap?.get(e.area_key)?.name;
      }
    });

    this.planAreas = [];
  }

  ngOnInit(): void {
    this.worldAreaService.getWorldAreas().subscribe(worldAreas => {
      this.areas = [...worldAreas.values()];
      this.areasMap = worldAreas;
    });
  }

  dropHandler(event: CdkDragDrop<WorldArea[]> | CdkDragDrop<PlanElement[]>) {
    if (event.previousContainer === event.container) {
      moveItemInArray(this.plan.plan, event.previousIndex, event.currentIndex);
    } else
      if (this.plan && this.areas) {
        this.plan.plan.splice(event.currentIndex, 0, this.planItemFromArea(this.areas[event.previousIndex]));
      }
  }

  planItemFromArea(area: WorldArea): PlanElement {
    return {
      area_key: area.named_id,
      notes: undefined,
    };
  }

  filterAreas(searchString: string) {
    if (searchString !== "") {

      return this.worldAreaService.matcher?.search(searchString).map(({ item }) => {
        return item[1];
      });
    } else {
      return this.areas;
    }
  }

  filterPlanElements(searchString: string) {
    if (searchString !== "") {
      return this.planFuzzer.search(searchString).map(({item}) => item);
    } else {
      return this.plan.plan;
    }
  }
}
