import { Injectable } from '@angular/core';
import { Config } from '../models/generated/Config';
import { invoke } from '@tauri-apps/api';
import { Subject, debounceTime } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class ConfigService {
  //We're making sure this is loaded initially so let's not indicate it being nullable.
  config!: Config;
  private syncSubject: Subject<void> = new Subject<void>();
  
  constructor() {
    // Let's do some premature optimization! Now we can do lots of config changes quickly from FE without as many sync calls :D
    this.syncSubject.pipe(debounceTime(500)).subscribe(() => this.underlyingSync());
  }

  init() {
    return invoke<Config>('load_config').then(cfg => {
      if(!cfg) {
        console.error("Could not load config, should generally not happen :')");
        return;
      }
      const _this = this;
      // Mostly to wrap setters so we can run the (debounced) sync function on any config changes!
      this.config = new Proxy(cfg, {
        get(target: any, property) {
          return target[property as keyof Config];
        },
        set(target: Config, property:any , value) {
          // target[property as keyof Config] = value;
          setConfig(target, property, value);
          _this.sync();
          return true;
        }
      });

    });
  }

  sync() {
    this.syncSubject.next();
  }

  private underlyingSync() {
    invoke('set_config', { config: this.config });
  }
}

function setConfig<T extends keyof Config>(target: Config, property: T, value: Config[T]) {
  target[property] = value;
}