import { Injectable, NgZone } from '@angular/core';
import { WorldArea } from '../models/world-area';
import { invoke } from '@tauri-apps/api';
import { Observable, ReplaySubject, Subject, filter, from, map } from 'rxjs';
import naturalCompare from 'natural-compare';
import { Fuzzr } from '../fuzzr/fuzzr';

@Injectable({
  providedIn: 'root'
})
export class WorldAreaService {
  private worldAreas?: Map<string, WorldArea>;
  public matcher?: Fuzzr;

  private worldAreasSubject = new ReplaySubject<Map<string, WorldArea>>();

  constructor(private zone: NgZone) {
    from(invoke<Map<string, WorldArea>>('load_world_areas')).subscribe((data) => {
      const entries = Object.entries(data).sort((a, b) => naturalCompare(a[1].named_id, b[1].named_id));
      this.worldAreas = new Map(entries);
      this.zone.run(() => this.worldAreasSubject.next(this.worldAreas!));
      this.matcher = new Fuzzr(this.worldAreas, {
        toString: (e: [string, WorldArea]) => e[1].name
      })
    });
  }

  getWorldAreas(): Observable<Map<string, WorldArea>> {
    return this.worldAreasSubject.asObservable().pipe(
      filter(worldAreas => !!worldAreas),
    );
  }  
}
